@@ -1,10 +1,11 @@
 #!/usr/bin/env python
 
 # ---------------------------------------------------------------------------
-# (C) 2008 Luca Beltrame (einar at heavensinferno dot net)
+# (C) 2008-2009 Luca Beltrame (einar at heavensinferno dot net)
 # (C) 2008 Giovanni Marco Dall'Olio (dalloliogm at gmail dot com)
 # This program is distributed under the terms of the GNU General Public
-# License (GPL), version 2. See the file COPYING for details.
+# License (GPL), version 2, or (at your option) any later version.
+# See the file COPYING for details.
 # ---------------------------------------------------------------------------
 
 "Module providing a Pythonic implementation of a data.frame."
@@ -14,11 +15,13 @@ from __future__ import division
 import os
 import csv
 import copy
+import itertools
+import warnings
 
 __author__ = "Luca Beltrame"
-__copyright__ = "Copyright 2008, Luca Beltrame, Giovanni Marco Dall'Olio"
-__license__ = "GPL (v2)"
-__version__ = "0.8"
+__copyright__ = "Copyright 2008-2009, Luca Beltrame, Giovanni Marco Dall'Olio"
+__license__ = "GPL (v2 or later)"
+__version__ = "0.9"
 __status__ = "Development"
 
 class DataMatrix(object):
@@ -58,8 +61,8 @@ class DataMatrix(object):
         - getRowByID - get a row with a specified row name
     """
 
-    def __init__(self,fh=None,row_names=None,header=True, delimiter="\t",
-                quoting=csv.QUOTE_MINIMAL, fixR=False):
+    def __init__(self, fh=None, row_names=None, header=True, delimiter="\t",
+                quoting=csv.QUOTE_MINIMAL, quotechar="'", fixR=False, skip=0):
 
         self._records = dict()
         self.rownames = list()
@@ -88,8 +91,15 @@ class DataMatrix(object):
 
         fh.seek(0) # to make sure
 
+        # Make a slice if we're skipping rows
+
+        if skip > 0:
+            data = itertools.islice(fh, skip, None)
+        else:
+            data = fh
+
         if header:
-            heading = fh.next()
+            heading = data.next()
             #FIXME: Possibly unsafe in certain situations
             heading = heading.strip()
             heading = heading.split(delimiter)
@@ -98,12 +108,23 @@ class DataMatrix(object):
             if fixR:
                 heading.insert(0,"x")
         else:
-            heading = fh.next()
+            heading = data.next()
             heading = heading.strip()
             heading = heading.split(delimiter)
             heading = ["Column " + str(index[0]+1)
                        for index in enumerate(heading)]
-            fh.seek(0)
+
+            #FIXME: Huge hack
+            if hasattr(data, "seek"):
+                # We are working on the file directly
+                data.seek(0)
+            else:
+                # We're on a slice, we need to reset
+                fh.seek(0)
+                data = itertools.islice(fh, skip, None)
+
+
+        # Slice the file handle if we need to skip
 
         self.columns = copy.copy(heading)
         count = 1
@@ -115,9 +136,9 @@ class DataMatrix(object):
         for field in self.columns:
             self._records[field] = list()
 
-        reader = csv.DictReader(fh,fieldnames=heading,
+        reader = csv.DictReader(data,fieldnames=heading,
                     delimiter=delimiter,quoting=quoting,
-                    restval="NA")
+                    restval="NA", quotechar=quotechar)
 
         for row in reader:
             if self.identifier is not None:
@@ -152,7 +173,7 @@ class DataMatrix(object):
 
         if key not in self._records:
             raise KeyError, unicode("No such column")
-        
+
         column_data = list()
 
         for record in self._records[key]:
@@ -190,6 +211,17 @@ class DataMatrix(object):
 
         self._records[key] = item
 
+    def _colIndex(self, column):
+
+        "Returns the index of a column. Returns -1 if not found."
+
+        if column in self:
+            for index, item in enumerate(self.columns):
+                if item == column:
+                    return index
+        else:
+            return -1
+
     def iterRows(self,**kwargs):
         """Iterate over a matrix's rows.
 
@@ -304,7 +336,6 @@ class DataMatrix(object):
                 "You must supply a sequence object to this method"))
 
         if len(other) != len(self):
-            print "Other", other
             raise ValueError(unicode("The column must be of the same length"))
         column_data = list()
         for index,item in enumerate(other):
@@ -406,19 +437,156 @@ class DataMatrix(object):
 
         self[colName] = colData
 
-def elementApply(matrix, func):
+    def rename(self, oldColumn, newColumn):
 
-    """Applies a function to each element of rows or columns, and outputs a new
-    matrix as result. If the function requires any type conversion, that must be done
-    by the user."""
+        "Renames a column."
+
+        if oldColumn not in self:
+            raise KeyError, unicode("Unknown column")
+
+        # Bail out gracefully if the names are equal
+        if oldColumn == newColumn:
+            return
+
+        for index, column in enumerate(self.columns):
+
+            if oldColumn == column:
+                self.columns[index] = newColumn
+
+        self._records[newColumn] = self._records[oldColumn]
+
+        del self._records[oldColumn]
+
+
+class EmptyMatrix(DataMatrix):
+
+        """DataMatrix variant that once instantiated generates an
+        empty matrix with specified columns and row names.
+        Does not depend upon reading a file.
+        Rows and columns, after initialization, can be added with insertRows,
+        insert, appendRows and append methods, respectively."""
+
+        def __init__(self, identifier = None, row_names = None, columns = None):
+
+            self.rownames = row_names if row_names else list()
+            self._records = dict()
+            self.columns = list() if not columns else columns
+            self.identifier = identifier
+
+            for column in self.columns:
+                self._records[column] = list()
+
+def cbind(*args):
+
+    """Joins a list of matrices by their columns. They must have the same length.
+    Additionally, the two matrices must not have equal column names.
+    Returns a DataMatrix instance of the joined result."""
+
+    args = list(args) # Make it mutable
+
+    final_matrix = args.pop(0)
+
+    for index, item in enumerate(args):
+        assert isinstance(item, DataMatrix), "Not a valid DataMatrix isntance"
+        final_matrix = _cbind(final_matrix, item)
+
+    return final_matrix
+
+def _cbind(first_matrix, second_matrix):
+
+    """Joins two matrices by their columns. They must have the same length.
+    Additionally, the two matrices must not have equal column names.
+    Returns a DataMatrix instance of the joined result."""
+
+    if len(first_matrix) != len(second_matrix):
+        raise ValueError, unicode("Row length mismatch")
+
+    #FIXME: Deal with header=False
+    duplicates = [item for item in first_matrix.columns if item in
+                  second_matrix.columns]
+    if duplicates:
+        raise ValueError, unicode("Duplicate column names are not supported.")
+
+    dest = EmptyMatrix(row_names=first_matrix.rownames)
+
+    for column in first_matrix:
+        data = first_matrix[column]
+        dest.append(data, column)
+    for column in second_matrix:
+        data = second_matrix[column]
+        dest.append(data, column)
+    return dest
+
+def rbind (first_matrix, second_matrix):
+
+    """Joins two matrices by their rows. The number of columns of the two
+    matrices must be identical. Returns a new DataMatrix instance with the
+    joined result."""
+
+    if len(first_matrix.columns) != len(second_matrix.columns):
+        raise ValueError, unicode(
+            "The number of columns of the two matrices is not equal")
+
+    count = len(first_matrix)
+    dest = EmptyMatrix(columns=first_matrix.columns)
+
+    for row in first_matrix.iterRows():
+        dest.appendRow(row[1:], row[0])
+
+    for row in second_matrix.iterRows():
+        if first_matrix.identifier is None and\
+           second_matrix.identifier is None:
+            row_name = str(count + int(row[0]))
+        else:
+            row_name = row[0]
+
+        dest.appendRow(row[1:], row_name)
+
+    return dest
+
+def apply(matrix, func, column, whole=False):
+
+    """Applies a function to a specific column. Rows are not supported (yet).
+    The operation is performed in-place. If the parameter "whole" is specified,
+    the function is applied to all the column at once, rather than to each of
+    its members."""
 
     assert isinstance(matrix, DataMatrix)
 
+    #FIXME: Return a copy, it's safer!
+
+    if column in matrix:
+        columnData = matrix[column]
+        if whole:
+            result = func(columnData)
+        else:
+            result = [func(item) for item in columnData]
+        matrix.replace(result, column)
+
+def elementApply(matrix, func, columns=None):
+
+    """Applies a function to each column for each row, and outputs a new
+    matrix as result. If the function requires any type conversion, that must be
+    done by the user. If the columns parameter is not None, the function is
+    applied only to specific columns."""
+
+    assert isinstance(matrix, DataMatrix)
+
+    if columns:
+        indices = [matrix._colIndex(item) for item in columns]
+
     resultMatrix = EmptyMatrix(columns = matrix.columns,
             identifier=matrix.identifier)
+
     for row in matrix.iterRows():
-        newRow = [func(item) for item in row[1:]]
-        resultMatrix.appendRow(newRow,row[0])
+        rowname = row[0]
+
+        if columns is not None:
+            for index in indices:
+                row[index+1] = func(row[index+1])
+        else:
+            row = [func(item) for item in row[1:]]
+        resultMatrix.appendRow(row, rowname)
 
     return resultMatrix
 
@@ -460,10 +628,10 @@ def matrixApply(matrix, func, what="rows", resultName="Function result"):
 
 def filterMatrix(matrix, func, column):
 
-    """Function which returns a DataMatrix with a column that satisfies specified
-    criteria. In particular, "func" must be a function applied to the each row
-    (on the column of interest) and should return
-    True if the row needs to be included, and False otherwise."""
+    """Function which returns a DataMatrix with a column that satisfies
+    specified criteria. In particular, "func" must be a function applied to
+    each row (on the column of interest) and should return True if the row
+    needs to be included, and False otherwise."""
 
     assert isinstance(matrix, DataMatrix)
 
@@ -531,15 +699,19 @@ def _median(values):
 
 def meanColumns(sourceMatrix):
 
-    "Calculates the mean of all columns for each row."
+    """Convenience function to calculate the mean of the columns of a
+    DataMatrix instance."""
+
+    return matrixApply(sourceMatrix, _mean,
+                       resultName="Mean value", what="columns")
 
-    return matrixApply(sourceMatrix, _mean, resultName="Mean value", what="columns")
-        
 def meanRows(sourceMatrix):
 
-    "Calculates the mean of all rows for each columns."
+    """Convenience function to calculate the mean of the rows of a DataMatrix
+    instance."""
 
-    return matrixApply(sourceMatrix, _mean, resultName="Mean value", what="rows")
+    return matrixApply(sourceMatrix, _mean, 
+                       resultName="Mean value", what="rows")
 
 def transpose(matrix, identifier="x"):
 
@@ -556,24 +728,21 @@ def transpose(matrix, identifier="x"):
     
     return destination
 
+def subset(matrix, column_list):
 
-class EmptyMatrix(DataMatrix):
+    """Creates a new matrix with only a subset of the columns present, as
+    specified by the columns_list parameter."""
 
-    """DataMatrix variant that once instantiated generates an
-    empty matrix with specified columns and row names.
-    Does not depend upon reading a file.
-    Rows and columns, after initialization, can be added with insertRows,
-    insert, appendRows and append methods, respectively."""
+    destination = EmptyMatrix(row_names=matrix.rownames,
+                              identifier=matrix.identifier)
+    for column in column_list:
+        if column in matrix:
+            destination.append(matrix[column], column)
 
-    def __init__(self, identifier = None, row_names = None, columns = None):
+    if not destination.columns:
+        warnings.warn("No columns selected")
 
-        self.rownames = row_names if row_names else list()
-        self._records = dict()
-        self.columns = list() if not columns else columns
-        self.identifier = identifier
-
-        for column in self.columns:
-            self._records[column] = list()
+    return destination
 
 def _test():
     import doctest
@@ -582,5 +751,6 @@ def _test():
     import test_datamatrix
     test_datamatrix.runTests()
 
+
 if __name__ == '__main__':
     _test()
